package de.mdelab.mlsdm.interpreter.executionTrace;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;

import de.mdelab.mlcore.MLElementWithUUID;
import de.mdelab.sdm.interpreter.core.executionTrace.ProfilingFilter;
import de.mdelab.sdm.interpreter.core.notifications.InterpreterNotification;

public class UuidBasedProfilingFiler implements ProfilingFilter<EClassifier>
{
	private final Set<String>				filteredUuids;

	private final Set<MLElementWithUUID>	filteredElements;

	private final boolean					includeChildren;

	public UuidBasedProfilingFiler(final String[] filteredUuids, final boolean includeChildren)
	{
		this.filteredElements = new HashSet<MLElementWithUUID>();
		this.filteredUuids = new HashSet<String>();

		this.includeChildren = includeChildren;

		for (final String uuid : filteredUuids)
		{
			this.filteredUuids.add(uuid);
		}
	}

	@Override
	public boolean doProfile(final InterpreterNotification<EClassifier> notification)
	{
		if (this.filteredElements.contains(notification.getMainStoryDiagramElement()))
		{
			return true;
		}
		else if (notification.getMainStoryDiagramElement() != null
				&& this.filteredUuids.contains(((MLElementWithUUID) notification.getMainStoryDiagramElement()).getUuid()))
		{
			this.filteredElements.add((MLElementWithUUID) notification.getMainStoryDiagramElement());

			if (this.includeChildren)
			{
				for (final Iterator<EObject> it = ((MLElementWithUUID) notification.getMainStoryDiagramElement()).eAllContents(); it
						.hasNext();)
				{
					final EObject eObject = it.next();

					if (eObject instanceof MLElementWithUUID)
					{
						this.filteredElements.add((MLElementWithUUID) eObject);
					}
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}
}
